--    Copyright (C) 2004 by neuron                                          
--    aagaande@gmail.com                                             
--                                                                          
--    This program is free software; you can redistribute it and--or modify  
--    it under the terms of the GNU General Public License as published by  
--    the Free Software Foundation; either version 2 of the License, or     
--    (at your option) any later version.                                   
--                                                                          
--    This program is distributed in the hope that it will be useful,       
--    but WITHOUT ANY WARRANTY; without even the implied warranty of        
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
--    GNU General Public License for more details.                          
--                                                                          
--    You should have received a copy of the GNU General Public License     
--    along with this program; if not, write to the                         
--    Free Software Foundation, Inc.,                                       
--    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 

-- TODO : Replace all Crayon:White with colors in layout?


local deformat = AceLibrary("Deformat-2.0")
local crayon = AceLibrary("Crayon-2.0")
local tourist = AceLibrary("Tourist-2.0")
local quixote = AceLibrary("Quixote-1.0")

function GetColor(r,g,b)
	local color = r
	if (g) then
		color = string.format("%02x%02x%02x", r*255, g*255, b*255)
	elseif (r["r"]) then
		g = r["g"]
		b = r["b"]
		r = r["r"]
		color = string.format("%02x%02x%02x", r, g, b)
	end
	--nQuestLog:Print("|c" .. color .. "Returning this color|r " .. color)
	return color
end


function nQuestLog:Format_QuestZone(strZone)
	local tmp_Zone = strZone
	local strLevelRange = ""

	-- Deal with the zone level:
	local iLow, iHigh = tourist:GetLevel(strZone)
	if (self.db.profile.showZoneLevel and iLow > 0 and iHigh > 0) then
		strLevelRange = strLevelRange .. " " .. iLow .. "-" .. iHigh
		
		local color = GetColor(self.db.profile.zoneHeader)
		if (self.db.profile.colorZoneLevelByLevel) then
			color = GetColor(tourist:GetLevelColor(strZone))
		end
		strLevelRange = crayon:Colorize(color,strLevelRange)
	end
	
	-- Deal with the zone text
	local color = GetColor(self.db.profile.zoneHeader)
	if (self.db.profile.colorZoneByLevel) then
		color = GetColor(tourist:GetLevelColor(strZone))
	end
	tmp_Zone = crayon:Colorize(color,tmp_Zone)
	
	return tmp_Zone .. strLevelRange
end


function nQuestLog:Format_QuestTitle(strTitle,iLevel,strTag,iTotalGot,iTotalNeeded,iComplete)
	local strNewTitle = "[" .. iLevel .. strTag .. "]" .. strTitle
	
	--Color quest name
	local color = GetColor(self.db.profile.questHeader)
	if (self.db.profile.colorQuestByLevel) then
		local tmp = GetDifficultyColor(iLevel)
		color = string.format("%02x%02x%02x", tmp.r*255, tmp.g*255, tmp.b*255)
	end
	strNewTitle = crayon:Colorize(color,strNewTitle)
	

	--Color quest progress
	local strProgress = ""
	if (iComplete == -1) then
		strProgress = crayon:White(" (failed)")
	else
		if (iTotalGot == iTotalNeeded) then
			if (iTotalNeeded == 0) then
				strProgress = crayon:White(" (goto)")
			else
				strProgress = crayon:White(" (done)")
			end
		else
			-- Todo make this an option if (self.db.profile.showQuestProgress and not self:ShouldShowObjective(strTitle,iTotalGot,iTotalNeeded)) then
			if (self.db.profile.showQuestProgress) then
				strProgress = " " .. iTotalGot .. "/" .. iTotalNeeded

				if (self.db.profile.colorQuestProgressByProgress) then
					local color = crayon:GetThresholdHexColor(iTotalGot,iTotalNeeded)
					strProgress = crayon:Colorize(color,strProgress)
				else
					strProgress = crayon:White(strProgress)
				end
			end
		end
	end
	return "  " .. strNewTitle .. strProgress
end



function nQuestLog:UpdateSmartObjectiveTracking()
	-- This function should be called before scanning through quest objectives
	if (not self.db.profile.smartObjectiveTracking) then
		return
	end

	-- self:Print("Doing smart quest tracking")

	if (self.db.profile.smartObjectiveTracking_MaxAge ~= 0) then
		local minTime = GetTime() - (self.db.profile.smartObjectiveTracking_MaxAge * 60)
		-- Remove quests that are too old
		for name,time in pairs(self.ObjectivesDB) do
			if time ~= true and time ~= false then
				if minTime >  time then
					-- self.ObjectivesDB[title] is time (ie not manual) and is too old, remove it
					self:Print("Quest objective " .. name .. " too old, hiding.")
					self.ObjectivesDB[name] = nil
				end
			end
		end
	end
	
	local function removeOldest()
		-- Function to always remove the oldest quest:
		local oldestName
		local oldestTime = GetTime()
		local questsTracked = 0
		
		for name,time in pairs(self.ObjectivesDB) do
			if time ~= true and time ~= false then
				if oldestTime > time then
					oldestTime = time
					oldestName = name
				end
				questsTracked = questsTracked + 1
			end
		end
		
		if (questsTracked > self.db.profile.smartObjectiveTracking_Max) then
			self:Print("Too many quest objectives tracked, removing monitoring for " .. oldestName)
			self.ObjectivesDB[oldestName] = nil
			questsTracked = questsTracked - 1
			if (questsTracked > self.db.profile.smartObjectiveTracking_Max) then
				removeOldest()
			end
		end
	end
	
	if (self.db.profile.smartObjectiveTracking_Max > 0) then
		removeOldest()
	end
end

function nQuestLog:ShouldShowObjective(strTitle, iGot, iNeeded,iForceObjectives)
	-- Returns true if we should show a specific objective (strTitle = quest title, not objective title)
	
	
	-- TODO, when scanning and removing old quests we should take zone into consideration, to hide out of zone quests before other quests?
	-- Can also take level into consideration, and let it weight a bit atleast
	-- prioritize stuff where we have 0/total over removing things where we have some objectives
	
	if (iForceObjectives) then
		return true
	end
	
	-- Function to check if quest objectives should be shown.
	if (self.ObjectivesDB[strTitle] == true or self.ObjectivesDB[strTitle] == false) then
		-- self.ObjectivesDB[strTitle] overrides every setting
		return self.ObjectivesDB[strTitle]
	end

	-- Hide completed objectives even with smart quest tracking
	if (self.db.profile.hideCompletedObjectives and iGot == iNeeded) then
		-- Hide completed
		return false
	end
	
	-- Smart quest tracking:
	if (self.db.profile.smartObjectiveTracking) then
		-- See ScanSmartObjectiveTracking(), if this isn't nil and smart objective tracking is on we should track this quest
		if (self.ObjectivesDB[strTitle] ~= nil) then
			return true
		end
		return false
	end
	
	if (self.db.profile.hideAllObjectives) then
		-- Hide everything, simple
		return false
	end
	
	return true
end

function nQuestLog:Format_QuestObjectives(strTitle,iObjectives,strObjective,iQuestId,iForceObjectives)
	-- Large function to format quest objectives
	local strObjectives = ""
	local iTotalGot, iTotalNeeded = 0,0
	
	local function Format_QuestObjective(strTitle,strObjective,strDesc, strType, iGot, iNeeded,iForceObjectives)
		-- Function to format a specific quest objective
		if not self:ShouldShowObjective(strTitle,iGot,iNeeded,iForceObjectives) then
			return ""
		end
		
		local tmpDesc = strDesc
		if (self.db.profile.colorObjectiveByProgress) then
			local color = crayon:GetThresholdHexColor(iGot,iNeeded)
			tmpDesc = crayon:Colorize(color,tmpDesc)
		else
			tmpDesc = crayon:White(tmpDesc)
		end
		
		local tmpProgress = iGot .. "/" .. iNeeded
		if (self.db.profile.colorObjectiveProgressByProgress) then
			local color = crayon:GetThresholdHexColor(iGot,iNeeded)
			tmpProgress = crayon:Colorize(color,tmpProgress)
		else
			tmpProgress = crayon:White(tmpProgress)
		end
		
		return "\n   - " .. tmpDesc .. " " .. tmpProgress
	end


	-- Loop through quest objectives and add them all
	if (iObjectives > 0) then
		for i=1,iObjectives do
			local strDesc, strType, iGot, iNeeded, isDone = quixote:GetQuestStatusById(iQuestId, i)
			if (strType == "reputation") then iNeeded = 1	iGot = 0	if (isDone) then iGot = 1 end end
			
			strObjectives = strObjectives .. Format_QuestObjective(strTitle,strObjective,strDesc, strType, iGot, iNeeded,iForceObjectives)
			
			iTotalGot = iTotalGot + iGot
			iTotalNeeded = iTotalNeeded + iNeeded
			
			
		end
	else
		if (not iForceObjectives and self:ShouldShowObjective(strTitle, iGot, iNeeded)) then
			strObjectives = "\n   - " .. crayon:Copper(strObjective)
		end
	end
	
	if (strObjectives == "" and self.ObjectivesDB[strTitle] and (self.ObjectivesDB[strTitle] ~= true or self.ObjectivesDB[strTitle] ~= false)) then
		-- Clear the timestamp, since the quest objectives are no longer being tracked.
		self.ObjectivesDB[strTitle] = nil
	end
	
	return strObjectives,iTotalGot,iTotalNeeded
end

